@@ -2,7 +2,7 @@ package Outthentic::DSL;
 
 use strict;
 
-our $VERSION = '0.0.8';
+our $VERSION = '0.0.9';
 
 use Carp;
 use Data::Dumper;
@@ -638,7 +638,7 @@ Language to validate text output.
 
 =head1 Informal introduction
 
-Alternative outthentic dsl introduction could be found here - L<intro.md|https://github.com/melezhik/outthentic-dsl/blob/master/intro.md>
+Alternative outthentic dsl introduction could be found L<here|https://github.com/melezhik/outthentic-dsl/blob/master/intro.md>
 
 
 =head1 Glossary
@@ -696,7 +696,7 @@ another program languages code
 
 =item *
 
-Is a language to verify I<arbitrary> plain text 
+Is a language to verify I<arbitrary> plain text
 
 
 
@@ -711,28 +711,26 @@ Outthentic dsl is both imperative and declarative language
 
 =head3 Declarative way
 
-You define rules ( check expressions ) to describe expected content
-
+You define rules ( check expressions ) to describe expected content.
 
 
 =head3 Imperative way
 
-You define a process of verification using custom perl code ( validators, generators, code expressions  ) 
-
+You define a process of verification using custom perl code ( validators, generators, code expressions  ).
 
 
 =head2 dsl code
 
-A  program code written on outthentic dsl language to verify text input
+A  program code written on outthentic dsl language to verify text input.
 
 
 =head2 Search context
 
-Verification process if carried out in a given context.
+Verification process is carried out in a given I<context>.
 
 But default search context is the same as original input text stream. 
 
-Search context could be however changes in some conditions
+Search context could be however changed in some conditions.
 
 
 =head2 dsl parser
@@ -769,12 +767,12 @@ Verification process consists of matching lines of text input against check expr
 This is schematic description of the process:
 
     For every check expression in check expressions list.
-        Mark this check step in unknown state.
+        Mark this check step as in unknown state.
         For every line in input text.
             Does line match check expression? Check step is marked as succeeded.
             Next line.
         End of loop.
-        Is this check step marked in unknown state? Mark this check step in failed state.  
+        Is this check step marked in unknown state? Mark this check step as in failed state.  
         Next check expression.
     End of loop.
     Are all check steps succeeded? Input text is verified.
@@ -782,7 +780,7 @@ This is schematic description of the process:
 
 A final I<presentation> of verification results should be implemented in a certain L<client|#clients> I<using> L<parser api|#parser-api> and not being defined at this scope. 
 
-For the sake of readability a I<fake> layout is used in this document. 
+For the sake of readability a I<fake> results presentation layout is used in this document. 
 
 
 =head2 Parser API
@@ -876,7 +874,7 @@ Default value is `0' - means do not create debug messages.
 
 =head3 validate
 
-Perform verification process 
+Perform verification process. 
 
 Obligatory parameter is:
 
@@ -1085,7 +1083,7 @@ Result - verified
 
 Parser does not care about I<how many times> a given check expression is matched in input text.
 
-If at least one line in a text match the check expression - I<this check> is considered as succeeded
+If at least one line in a text match the check expression - I<this check> is considered as succeeded.
 
 Parser  I<accumulate> all matching lines for given check expression, so they could be processed.
 
@@ -1183,7 +1181,7 @@ Input text:
     with that string
     at the very end.
 
-Result - not verified
+Result - verified
 
 Input text:
 
@@ -1311,7 +1309,7 @@ Dsl code:
     validator: [ 1>10, 'one is bigger then ten'  ]
     
     # this one depends on previous check
-    regexp: card number: (\d+)
+    regexp: credit card number: (\d+)
     validator: [ captures()->[0]-[0] == '0101010101', 'I know your secrets!'  ]
     
     
@@ -1352,7 +1350,7 @@ The only requirement for generator code - it should return I<reference to array
 
 Strings in array returned by generator code I<represent> a I<new> outthentic entities.
 
-A new outthentic entries are passed back to parser and executed immediately
+A new outthentic entries are passed back to parser and executed immediately.
 
 Generators expressions start with `:generator' marker.
 
@@ -1465,7 +1463,7 @@ Dsl code:
     string
     here
 
-Verification results:
+Results:
 
     +--------+---------------------------------------+
     | status | message                               |
@@ -1682,7 +1680,7 @@ thus next plain string checks expression will be executed against new search con
 
 =back
 
-The result will be:
+Results:
 
     +--------+------------------------------------------------+
     | status | message                                        |
@@ -1710,32 +1708,33 @@ Within expressions could be sequential, which effectively means using `&&' logic
     # and try to find month 04 in a date string
     within: \d\d\d\d-04-\d\d
 
-Speaking in human language chained within expressions acts like specifications. When you may start with some
-generic assumptions and then make your requirements more specific. A failure on any step of chain results in
+Speaking in human language chained within expressions acts like I<specifications>. 
+
+When you may start with some generic assumptions and then make your requirements more specific. A failure on any step of chain results in
 immediate break. 
 
 
 =head1 Range expressions
 
-Between or range expressions also act like I<search context modificators> - they change search area to one included
+Range expressions also act like I<search context modificators> - they change search area to one included
 I<between> lines matching right and left regular expression of between statement.
 
 It is very similar to what Perl L<range operator|http://perldoc.perl.org/perlop.html#Range-Operators> does 
-when extracting pieces of lines inside stream
+when extracting pieces of lines inside stream:
 
     while (<STDOUT>){
         if /foo/ ... /bar/
     }
 
-Outthentic analogy for this is between expression:
+Outthentic analogy for this is range expression:
 
     between: foo bar
 
-Between expression takes 2 arguments - left and right regular expression to setup search area boundaries.
+Between statement takes 2 arguments - left and right regular expression to setup search area boundaries.
 
 A search context will be all the lines included between line matching left expression and line matching right expression.
 
-A matching (boundary) lines are not included in range:
+A matching (boundary) lines are not included in range. 
 
 These are few examples:
 
@@ -1765,8 +1764,10 @@ Dsl code:
     between: <tr.*> <\/tr>
     regexp: <td>(\S+)<\/td>
 
-Between expressions could not be nested, every new between expression discards old search context
-and setup new one:
+
+=head2 Multiple range expressions
+
+Multiple range expressions could not be nested, every new between statement discards old search context and setup new one:
 
 Input text:
 
@@ -1872,7 +1873,10 @@ Dsl code:
 =head2 Range expressions caveats
 
 
-=head3 Range expressions don't verify continuous list:
+=head3 Range expressions can't verify continuous lists.
+
+That means range expression only verifies that there are I<some set> of lines inside some range.
+It is not necessary should be continuous.
 
 Example.
 
@@ -1886,17 +1890,22 @@ Input text:
         3
         c
     bar
-    
-    
-    # outthentic check
-    
+
+Dsl code:
+
     between: foo bar
-        regexp: 1
-        code: print '#', ( join ' ', map {$_->[0]} @{captures()} ), "\n"
+        1
+        code: print capture()->[0], "\n"
+        2
+        code: print capture()->[0], "\n"
+        3
+        code: print capture()->[0], "\n"
 
-Output
+Output:
 
-        # 1 2 3 
+        1 
+        2 
+        3 
 
 If you need check continuous sequences checks use text blocks.
 
@@ -1908,7 +1917,7 @@ Below is highly experimental features purely tested. You may use it on your own
 
 =head2 Streams
 
-Streams are alternative for captures. Consider following example:
+Streams are alternative for captures. Consider following example.
 
 Input text:
 
@@ -1959,18 +1968,21 @@ Output:
 Notice something interesting? Output direction has been inverted.
 
 The reason for this is outthentic check expression works in "line by line scanning" mode 
-when output gets verified line by line against given check expression. Once all lines are matched
-they get dropped into one heap without preserving original "group context". 
+when text input gets verified line by line against given check expression. 
 
-What if we would like to print all matching lines grouped by text blocks they belong to which is more convenient?
+Once all lines are matched they get dropped into one heap without preserving original "group context". 
+
+What if we would like to print all matching lines grouped by text blocks they belong to?
+
+As it's more convenient way ...
 
 This is where streams feature comes to rescue.
 
 Streams - are all the data successfully matched for given I<group context>. 
 
-Streams are available for text blocks and range expressions.
+Streams are I<applicable> for text blocks and range expressions.
 
-Let's rewrite the example.
+Let's rewrite last example.
 
 Dsl code:
 
@@ -1993,17 +2005,21 @@ Dsl code:
         
     end:
 
-Stream function returns an arrays of streams. Every stream holds all the matched lines for given block.
-So streams preserve group context. Number of streams relates to the number of successfully matched blocks.
+Stream function returns an arrays of I<streams>. Every stream holds all the matched lines for given I<logical block>.
 
-Streams data presentation is much closer to what was originally given in stdout:
+Streams preserve group context. Number of streams relates to the number of successfully matched groups.
+
+Streams data presentation is much closer to what was originally given in text input:
+
+Output:
 
     # foo a b  c    bar
     # foo 1 2  3    bar
     # foo 0 00 000  bar
 
-Stream could be specially useful when combined with range expressions where sizes
-of successfully matched blocks could be different:
+Stream could be specially useful when combined with range expressions of I<various> ranges lengths.
+
+For example.
 
 Input text:
 
@@ -2018,6 +2034,12 @@ Input text:
         1
         3
     bar
+    
+    foo
+        0
+        0
+        0
+    bar
 
 Dsl code:
 
@@ -2038,6 +2060,7 @@ Output:
 
     # 2 4 6 8
     # 1 3
+    # 0 0 0
 
 
 =head1 Author
